import pytest
import rastro
import math

def test_deg2rad():
    assert rastro.DEG2RAD == math.pi/180.0

def test_DEG2RAD():
    assert rastro.DEG2RAD == math.pi/180.0

def test_RAD2DEG():
    assert rastro.RAD2DEG == 180.0/math.pi

def test_AS2RAD():
    assert rastro.AS2RAD == rastro.DEG2RAD / 3600.0

def test_RAD2AS():
    assert rastro.RAD2AS == rastro.RAD2DEG * 3600.0

def test_MJD_ZERO():
    assert rastro.MJD_ZERO == 2400000.5

def test_MJD2000():
    assert rastro.MJD2000 == 51544.5

def test_GPS_TAI():
    assert rastro.GPS_TAI == -19.0

def test_TAI_GPS ():
    assert rastro.TAI_GPS  == -rastro.GPS_TAI

def test_TT_TAI():
    assert rastro.TT_TAI == 32.184

def test_TAI_TT():
    assert rastro.TAI_TT == -rastro.TT_TAI

def test_GPS_TT():
    assert rastro.GPS_TT == rastro.GPS_TAI + rastro.TAI_TT

def test_TT_GPS():
    assert rastro.TT_GPS == -rastro.GPS_TT

def test_GPS_ZERO():
    assert rastro.GPS_ZERO == 44244.0

def test_C_LIGHT():
    assert rastro.C_LIGHT == 299792458.0

def test_AU():
    assert rastro.AU == 1.49597870700e11

def test_R_EARTH():
    assert rastro.R_EARTH == 6.378136300e6

def test_WGS84_A():
    assert rastro.WGS84_A == 6378137.0

def test_WGS84_F():
    assert rastro.WGS84_F == 1.0/298.257223563

def test_GM_EARTH():
    assert rastro.GM_EARTH == 3.986004415e14

def test_ECC_EARTH():
    assert rastro.ECC_EARTH == 8.1819190842622e-2

def test_J2_EARTH():
    assert rastro.J2_EARTH == 0.0010826358191967

def test_OMEGA_EARTH():
    assert rastro.OMEGA_EARTH == 7.292115146706979e-5

def test_GM_SUN():
    assert rastro.GM_SUN == 132712440041.939400*1e9

def test_R_SUN():
    assert rastro.R_SUN == 6.957*1e8

def test_P_SUN():
    assert rastro.P_SUN == 4.560E-6

def test_R_MOON():
    assert rastro.R_MOON == 1738*1e3

def test_GM_MOON():
    assert rastro.GM_MOON == 4902.800066*1e9

def test_GM_MERCURY():
    assert rastro.GM_MERCURY == 22031.780000*1e9

def test_GM_VENUS():
    assert rastro.GM_VENUS == 324858.592000*1e9

def test_GM_MARS():
    assert rastro.GM_MARS == 42828.37521*1e9

def test_GM_JUPITER():
    assert rastro.GM_JUPITER == 126712764.8*1e9

def test_GM_SATURN():
    assert rastro.GM_SATURN == 37940585.2*1e9

def test_GM_URANUS():
    assert rastro.GM_URANUS == 5794548.6*1e9

def test_GM_NEPTUNE():
    assert rastro.GM_NEPTUNE == 6836527.100580*1e9

def test_GM_PLUTO():
    assert rastro.GM_PLUTO == 977.000000*1e9



import pytest
import math
import rastro

def test_orbital_period():
    T = rastro.orbital_period(rastro.R_EARTH + 500e3)
    assert T == pytest.approx(5676.977164028288, abs=1e-12)

def test_orbital_period_general():
    a  = rastro.R_EARTH + 500e3
    gm = rastro.GM_EARTH
    T  = rastro.orbital_period_general(a, gm)

    assert T == pytest.approx(5676.977164028288, abs=1e-12)

def test_orbital_period_general_moon():
    a  = rastro.R_MOON + 500e3
    gm = rastro.GM_MOON
    T  = rastro.orbital_period_general(a, gm)
    
    assert T == pytest.approx(9500.531451174307, abs=1e-12)

def test_mean_motion():
    n = rastro.mean_motion(rastro.R_EARTH + 500e3, as_degrees=False)
    assert n == pytest.approx(0.0011067836148773837, abs=1e-12)

    n = rastro.mean_motion(rastro.R_EARTH + 500e3, as_degrees=True)
    assert n == pytest.approx(0.0634140299667068, abs=1e-12)

    n = rastro.mean_motion(rastro.R_EARTH + 500e3)
    assert n == pytest.approx(0.0634140299667068, abs=1e-12)

def test_mean_motion_general():
    n = rastro.mean_motion_general(rastro.R_EARTH + 500e3, rastro.GM_MOON, as_degrees=False)
    assert n != pytest.approx(0.0011067836148773837, abs=1e-12)

    n = rastro.mean_motion_general(rastro.R_EARTH + 500e3, rastro.GM_MOON, as_degrees=True)
    assert n != pytest.approx(0.0634140299667068, abs=1e-12)

def test_semimajor_axis():
    a = rastro.semimajor_axis(0.0011067836148773837, as_degrees=False)
    assert a == pytest.approx(rastro.R_EARTH + 500e3, abs=1e-6)

    a = rastro.semimajor_axis(0.0634140299667068, as_degrees=True)
    assert a == pytest.approx(rastro.R_EARTH + 500e3, abs=1e-6)

    a = rastro.semimajor_axis(0.0634140299667068)
    assert a == pytest.approx(rastro.R_EARTH + 500e3, abs=1e-6)

def test_semimajor_axis():
    a = rastro.semimajor_axis_general(0.0011067836148773837, rastro.GM_MOON, as_degrees=False)
    assert a != pytest.approx(rastro.R_EARTH + 500e3, abs=1e-6)

    a = rastro.semimajor_axis_general(0.0634140299667068, rastro.GM_MOON, as_degrees=True)
    assert a != pytest.approx(rastro.R_EARTH + 500e3, abs=1e-6)

def test_perigee_velocity():
    vp = rastro.perigee_velocity(rastro.R_EARTH + 500e3, 0.001)
    assert vp == pytest.approx(7620.224976404526, abs=1e-12)

def test_perigee_velocity_general():
    vp = rastro.periapsis_velocity(rastro.R_MOON + 500e3, 0.001, rastro.GM_MOON)
    assert vp == pytest.approx(1481.5842246768275, abs=1e-12)

def test_apogee_velocity():
    vp = rastro.apogee_velocity(rastro.R_EARTH + 500e3, 0.001)
    assert vp == pytest.approx(7604.999751676446, abs=1e-12)

def test_apogee_velocity_general():
    vp = rastro.apoapsis_velocity(rastro.R_MOON + 500e3, 0.001, rastro.GM_MOON)
    assert vp == pytest.approx(1478.624016435715, abs=1e-12)

def test_sun_synchronous_inclination():
    vp = rastro.sun_synchronous_inclination(rastro.R_EARTH + 500e3, 0.001, True)
    assert vp == pytest.approx(97.40172901366881, abs=1e-12)

def test_anomaly_eccentric_to_mean():
    # 0 
    M = rastro.anomaly_eccentric_to_mean(0.0, 0.0, as_degrees=False)
    assert M == 0

    M = rastro.anomaly_eccentric_to_mean(0.0, 0.0, as_degrees=True)
    assert M == 0

    # 180
    M = rastro.anomaly_eccentric_to_mean(math.pi/2, 0.1, as_degrees=False)
    assert M == pytest.approx(1.4707963267948965, abs=1e-12)

    M = rastro.anomaly_eccentric_to_mean(90.0, 0.1, as_degrees=True)
    assert M == pytest.approx(84.27042204869177, abs=1e-3)

    # 180
    M = rastro.anomaly_eccentric_to_mean(math.pi, 0.0, as_degrees=False)
    assert M == pytest.approx(math.pi, abs=1e-12)

    M = rastro.anomaly_eccentric_to_mean(180.0, 0.0, as_degrees=True)
    assert M == 180.0

def test_anomaly_mean_to_eccentric():
    # 0 
    E = rastro.anomaly_mean_to_eccentric(0.0, 0.0, as_degrees=False)
    assert E == 0

    E = rastro.anomaly_mean_to_eccentric(0.0, 0.0, as_degrees=True)
    assert E == 0

    # 180
    E = rastro.anomaly_mean_to_eccentric(1.4707963267948965, 0.1, as_degrees=False)
    assert E == pytest.approx(math.pi/2, abs=1e-12)

    E = rastro.anomaly_mean_to_eccentric(84.27042204869177, 0.1, as_degrees=True)
    assert E == pytest.approx(90.0, abs=1e-12)

    # 180
    E = rastro.anomaly_mean_to_eccentric(math.pi, 0.0, as_degrees=False)
    assert E == pytest.approx(math.pi, abs=1e-12)

    E = rastro.anomaly_mean_to_eccentric(180.0, 0.0, as_degrees=True)
    assert E == 180.0

    # Large Eccentricities
    E = rastro.anomaly_mean_to_eccentric(180.0, 0.9, as_degrees=True)
    assert E == 180.0

def test_anomaly_true_to_eccentric():
    # 0 degrees
    anm_ecc = rastro.anomaly_true_to_eccentric(0.0, 0.0, False)
    assert anm_ecc == 0.0

    anm_ecc = rastro.anomaly_true_to_eccentric(0.0, 0.0, True)
    assert anm_ecc == 0.0

    # 180 degrees
    anm_ecc = rastro.anomaly_true_to_eccentric(math.pi, 0.0, False)
    assert anm_ecc == math.pi

    anm_ecc = rastro.anomaly_true_to_eccentric(180.0, 0.0, True)
    assert anm_ecc == 180.0

    # 90 degrees
    anm_ecc = rastro.anomaly_true_to_eccentric(math.pi / 2.0, 0.0, False)
    assert anm_ecc == pytest.approx(math.pi/2.0, abs=1e-12);

    anm_ecc = rastro.anomaly_true_to_eccentric(90.0, 0.0, True)
    assert anm_ecc == pytest.approx(90.0, abs=1e-12);

    anm_ecc = rastro.anomaly_true_to_eccentric(math.pi / 2.0, 0.1, False)
    assert anm_ecc == pytest.approx(1.4706289056333368, abs=1e-12)

    anm_ecc = rastro.anomaly_true_to_eccentric(90.0, 0.1, True)
    assert anm_ecc == pytest.approx(84.26082952273322, abs=1e-12)

def test_anomaly_eccentric_to_true():
    # 0 degrees
    anm_true = rastro.anomaly_eccentric_to_true(0.0, 0.0, False)
    assert anm_true == 0.0

    anm_true = rastro.anomaly_eccentric_to_true(0.0, 0.0, True)
    assert anm_true == 0.0

    # 180 degrees
    anm_true = rastro.anomaly_eccentric_to_true(math.pi, 0.0, False)
    assert anm_true == math.pi

    anm_true = rastro.anomaly_eccentric_to_true(180.0, 0.0, True)
    assert anm_true == 180.0

    # 90 degrees
    anm_true = rastro.anomaly_eccentric_to_true(math.pi/2.0, 0.0, False)
    assert anm_true == pytest.approx(math.pi/2.0, abs=1e-12);

    anm_true = rastro.anomaly_eccentric_to_true(90.0, 0.0, True)
    assert anm_true == pytest.approx(90.0, abs=1e-12);

    anm_true = rastro.anomaly_eccentric_to_true(math.pi/2.0, 0.1, False)
    assert anm_true == pytest.approx(1.6709637479564563, abs=1e-12)

    anm_true = rastro.anomaly_eccentric_to_true(90.0, 0.1, True)
    assert anm_true == pytest.approx(95.73917047726677, abs=1e-12)

def test_anomaly_true_to_mean():
    # 0 degrees
    m = rastro.anomaly_true_to_mean(0.0, 0.0, False)
    assert m == 0.0

    m = rastro.anomaly_true_to_mean(0.0, 0.0, True)
    assert m == 0.0

    # 180 degrees
    m = rastro.anomaly_true_to_mean(math.pi, 0.0, False)
    assert m == math.pi

    m = rastro.anomaly_true_to_mean(180.0, 0.0, True)
    assert m == 180.0

    # 90 degrees
    m = rastro.anomaly_true_to_mean(math.pi / 2.0, 0.1, False)
    assert m == pytest.approx(1.3711301619226748, abs=1e-12)

    m = rastro.anomaly_true_to_mean(90.0, 0.1, True)
    assert m == pytest.approx(78.55997144125844, abs=1e-12)

def test_anomaly_mean_to_true():
    # 0 degrees
    nu = rastro.anomaly_mean_to_true(0.0, 0.0, False)
    assert nu == 0.0

    nu = rastro.anomaly_mean_to_true(0.0, 0.0, True)
    assert nu == 0.0

    # 180 degrees
    nu = rastro.anomaly_mean_to_true(math.pi, 0.0, False)
    assert nu == math.pi

    nu = rastro.anomaly_mean_to_true(180.0, 0.0, True)
    assert nu == 180.0

    # 90 degrees
    nu = rastro.anomaly_mean_to_true(math.pi/2.0, 0.1, False)
    assert nu == pytest.approx(1.7694813731148669, abs=1e-12)

    nu = rastro.anomaly_mean_to_true(90.0, 0.1, True)
    assert nu == pytest.approx(101.38381460649556, abs=1e-12)


#!/usr/bin/env python
import os

from setuptools import setup
from setuptools_rust import RustExtension

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="rastro",
    version="0.1.6", # Do NOT edit. Will be updated for release by CI pipeline
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Rust",
    ],
    packages=["rastro"],
    rust_extensions=[
        RustExtension("rastro_python.constants"),
        RustExtension("rastro_python.orbits")
    ],
    include_package_data=True,
    zip_safe=False,
    long_description=long_description,
    long_description_content_type='text/markdown',
)